from test_all_fixers import lib3to2FixerTestCase

class Test_imports2(lib3to2FixerTestCase):
    fixer = "imports2"

    def test_simple_imports(self):

        b = "from urllib.request import urlopen"
        a = "from urllib2 import urlopen"
        self.check(b, a)

        b = "from urllib.request import urlopen\n"\
            "from urllib.parse import urlencode"
        a = "from urllib2 import urlopen\n"\
            "from urllib import urlencode"
        self.check(b, a)

        b = "from tkinter.simpledialog import Grid, SimpleDialog"
        a = "from tkSimpleDialog import Grid\n"\
            "from SimpleDialog import SimpleDialog"
        self.check(b, a)
        
    def test_nested_import_all(self):
        
        b = """
        def try_import(package):
            try:
                from http.server import *
                print('success')
            except ImportError:
                print('failure', end="")
                print('try again!')
        """
        a = """
        def try_import(package):
            try:
                from CGIHTTPServer import *
                from SimpleHTTPServer import *
                from BaseHTTPServer import *
                print('success')
            except ImportError:
                print('failure', end="")
                print('try again!')
        """
        self.check(b, a, ignore_warnings=True)
        
        b = """
        def testing_http_server():
            from http.server import *
            test_all_imports()
        def testing_xmlrpc_server():
            from xmlrpc.server import *
            test_all_imports()
        """
        a = """
        def testing_http_server():
            from CGIHTTPServer import *
            from SimpleHTTPServer import *
            from BaseHTTPServer import *
            test_all_imports()
        def testing_xmlrpc_server():
            from SimpleXMLRPCServer import *
            from DocXMLRPCServer import *
            test_all_imports()
        """
        self.check(b, a, ignore_warnings=True)

    def test_nested_import_named(self):

        b = """
        with open('myFile', 'r') as myFile:
            from urllib.request import install_opener, urlretrieve
            fileList = [ln for ln in myFile]"""
        a = """
        with open('myFile', 'r') as myFile:
            from urllib import urlretrieve
            from urllib2 import install_opener
            fileList = [ln for ln in myFile]"""
        self.check(b, a, ignore_warnings=True)

    def test_alt_wording(self):

        b = """
        if spam.is_good():
            from urllib import request, parse
            request.urlopen(spam_site)
            parse.urlencode(spam_site)"""
        a = """
        if spam.is_good():
            import urllib
            import urllib2
            urllib2.urlopen(spam_site)
            urllib.urlencode(spam_site)"""
        self.check(b, a)

    def test_simplest_usage(self):

        b = """
        import urllib.request
        urllib.request.urlopen(spam)"""

        a = """
        import urllib2
        urllib2.urlopen(spam)"""

        self.check(b, a)



"""
Fixer for division: from __future__ import division if needed
"""

from lib2to3 import fixer_base
from lib2to3.pytree import Node
from lib2to3.fixer_util import Name, FromImport, Newline
from lib2to3.pgen2 import token

def match_division(node):
    """
    __future__.division redefines the meaning of a single slash for division,
    so we match that and only that.
    """
    slash = token.SLASH
    return node.type == slash and not node.next_sibling.type == slash and \
                                  not node.prev_sibling.type == slash

class FixDivision(fixer_base.BaseFix):

    def start_tree(self, tree, filename):
        """This is only run once; we want to remember the first node"""
        super(FixDivision, self).start_tree(tree, filename)
        self._tree = tree
        self.have_div = False

    def match(self, node):
        """
        Since the tree needs to be fixed once and only once if and only if it
        matches, then we can start discarding matches after we make the first.
        """
        return not self.have_div and match_division(node)

    def transform(self, node, results):
        tree = self._tree
        self.have_print = True
        syms = self.syms
        future_stmt = FromImport(u"__future__",
                                [Name(u"division", prefix=u" ")])
        children = tree.children[:]
        new_node = Node(syms.simple_stmt, [future_stmt, Newline()])
        for child in children:
            child.remove()
            new_node.append_child(child)
        tree.insert_child(0, new_node)



from lib2to3.fixer_util import Comma

def commatize(leafs):
    """
    Accepts/turns: (Name, Name, ..., Name, Name) 
    Returns/into: (Name, Comma, Name, Comma, ..., Name, Comma, Name)
    """
    new_leafs = []
    for leaf in leafs:
        new_leafs.append(leaf)
        new_leafs.append(Comma())
    del new_leafs[-1]
    return new_leafs



"""
Fixer to alert the user about features in 3.x that do not exist in 2.5
Any matches here will necessarily constrain the input code to work after refactoring
only in later versions of Python than 2.5.
"""

from lib2to3 import fixer_base
from lib2to3.fixer_util import does_tree_import, is_import

def incompatible_feature(obj, node, feature):
    reason = "The following is not backwards compatible with Python 2.5 and will only run in version %s and up: %s" %(feature.min_ver, feature.name)
    obj.cannot_convert(node, reason)

class Feature(object):

    def __init__(self, name, min_ver):
        self.name = name
        self.min_ver = min_ver

class FixFeatures(fixer_base.BaseFix):

    # Run this as late as possible, after optionally fixed features are taken care of.
    run_order = 9
    #TODO: Make the pattern built generically from a mapping.
    PATTERN = """format=power< STRING trailer< '.' 'format' > trailer< '(' any* ')' > > |
                 bin=power< 'bin' trailer< '(' any* ')' > [any*] > |
                 io=import_name< 'import' ('io' | dotted_as_names< any* 'io' any* >) > |
                 io=import_from< 'from' 'io' 'import' any* > |
                 abc=import_name< 'import' ('abc' | dotted_as_names< any* 'abc' any* >) > |
                 abc=import_from< 'from' 'abc' 'import' any* > |
                 numb=import_from< 'from' 'numbers' 'import' any* > |
                 numb=import_name< 'import' ('numbers' | dotted_as_names< any* 'numbers' any* >) > |
                 mem=power< 'memoryview' trailer< '(' any* ')' > [any*] >
              """
    def handle_imports(self, node):
        #TODO: Make the handler generic.
        found = []
        if does_tree_import(None, 'io', node) or \
           (is_import(node) and ' io ' in str(node)):
            found.append(Feature('io package', '2.6'))
        if does_tree_import(None, 'abc', node) or \
           (is_import(node) and ' abc ' in str(node)):
            found.append(Feature('abc package', '2.6'))
        if does_tree_import(None, 'numbers', node) or \
           (is_import(node) and ' numbers ' in str(node)):
            found.append(Feature('numbers package', '2.6'))
        return found

    def transform(self, node, results):
        #TODO: Make the transformation generic.
        def alert_feature(feature):
            return incompatible_feature(self, node, feature)
        if results.get('bin'):
            alert_feature(Feature('bin function', '2.6'))
        if results.get('mem'):
            alert_feature(Feature('memoryview function', '2.7'))
        io = results.get('io')
        abc = results.get('abc')
        numb = results.get('numb')
        if io or numb or abc:
            for found in self.handle_imports(node):
                alert_feature(found)



"""
Fixer for:
functools.reduce(f, it) -> reduce(f, it)
from functools import reduce -> (remove this line)
"""

from lib2to3 import fixer_base
from lib2to3.fixer_util import Call
from lib2to3.pytree import Node, Leaf
from lib2to3.pgen2 import token

class FixReduce(fixer_base.BaseFix):

    PATTERN = """
    power< 'functools' trailer< '.' 'reduce' >
                                    args=trailer< '(' arglist< any* > ')' > > |
    imported=import_from< 'from' 'functools' 'import' 'reduce' >
    """

    def transform(self, node, results):
        syms = self.syms
        args, imported = (results.get("args"), results.get("imported"))
        if imported:
            imported.remove()
        elif args:
            args = args.clone()
            prefix = node.prefix
            return Node(syms.power, [Leaf(token.NAME, u"reduce"), args],
                                                                 prefix=prefix)



from test_all_fixers import lib3to2FixerTestCase

class Test_bytes(lib3to2FixerTestCase):
    fixer = "bytes"

    def test_bytes_call_1(self):
        b = """bytes(x, y, z)"""
        a = """str(x, y, z)"""
        self.check(b, a)

    def test_bytes_call_2(self):
        b = """bytes(x, y, z) + b"florist" """
        a = """str(x, y, z) + "florist" """
        self.check(b, a)

    def test_bytes_literal_1(self):
        b = '''b"\x41"'''
        a = '''"\x41"'''
        self.check(b, a)

    def test_bytes_literal_2(self):
        b = """b'x'"""
        a = """'x'"""
        self.check(b, a)

    def test_bytes_literal_3(self):
        b = """BR'''\x13'''"""
        a = """R'''\x13'''"""
        self.check(b, a)

    def test_bytes_concatenation(self):
        b = """b'bytes' + b'bytes'"""
        a = """'bytes' + 'bytes'"""
        self.check(b, a)
